import textwrap
from datetime import datetime
from contextlib import contextmanager

import praw
import six

from .errors import SubmissionURLError, SubredditNameError

def clean(unicode_string):
    """
    Convert unicode string into ascii-safe characters.
    """

    if six.PY2:
        ascii_string = unicode_string.encode('ascii', 'replace')
    else:
        ascii_string = unicode_string.encode().decode('ascii', 'replace')

    ascii_string = ascii_string.replace('\\', '')
    return ascii_string


def strip_subreddit_url(permalink):
    """
    Grab the subreddit from the permalink because submission.subreddit.url
    makes a seperate call to the API.
    """

    subreddit = clean(permalink).split('/')[4]
    return '/r/{}'.format(subreddit)


def humanize_timestamp(utc_timestamp, verbose=False):
    """
    Convert a utc timestamp into a human readable relative-time.
    """

    timedelta = datetime.utcnow() - datetime.utcfromtimestamp(utc_timestamp)

    seconds = int(timedelta.total_seconds())
    if seconds < 60:
        return 'moments ago' if verbose else '0min'
    minutes = seconds // 60
    if minutes < 60:
        return ('%d minutes ago' % minutes) if verbose else ('%dmin' % minutes)
    hours = minutes // 60
    if hours < 24:
        return ('%d hours ago' % hours) if verbose else ('%dhr' % hours)
    days = hours // 24
    if days < 30:
        return ('%d days ago' % days) if verbose else ('%dday' % days)
    months = days // 30.4
    if months < 12:
        return ('%d months ago' % months) if verbose else ('%dmonth' % months)
    years = months // 12
    return ('%d years ago' % years) if verbose else ('%dyr' % years)

@contextmanager
def default_loader(self):
    yield

class BaseContent(object):

    def get(self, index, n_cols):
        raise NotImplementedError

    def iterate(self, index, step, n_cols):

        while True:

            # Hack to prevent displaying negative indicies if iterating in the
            # negative direction.
            if step < 0 and index < 0:
                break

            try:
                yield self.get(index, n_cols=n_cols)
            except IndexError:
                break
            index += step

    @staticmethod
    def flatten_comments(comments, root_level=0):
        """
        Flatten a PRAW comment tree while preserving the nested level of each
        comment via the `nested_level` attribute.
        """

        stack = comments[:]
        for item in stack:
            item.nested_level = root_level

        retval = []
        while stack:
            item = stack.pop(0)
            nested = getattr(item, 'replies', None)
            if nested:
                for n in nested:
                    n.nested_level = item.nested_level + 1
                stack[0:0] = nested
            retval.append(item)
        return retval

    @staticmethod
    def strip_praw_comment(comment):
        """
        Parse through a submission comment and return a dict with data ready to
        be displayed through the terminal.
        """

        data = {}
        data['object'] = comment
        data['level'] = comment.nested_level

        if isinstance(comment, praw.objects.MoreComments):
            data['type'] = 'MoreComments'
            data['count'] = comment.count
            data['body'] = 'More comments'.format(comment.count)
        else:
            data['type'] = 'Comment'
            data['body'] = clean(comment.body)
            data['created'] = humanize_timestamp(comment.created_utc)
            data['score'] = '{} pts'.format(comment.score)
            data['author'] = (clean(comment.author.name) if
                              getattr(comment, 'author') else '[deleted]')

            sub_author = (clean(comment.submission.author.name) if
                          getattr(comment.submission, 'author') else '[deleted]')
            data['is_author'] = (data['author'] == sub_author)

        return data

    @staticmethod
    def strip_praw_submission(sub):
        """
        Parse through a submission and return a dict with data ready to be
        displayed through the terminal.
        """

        is_selfpost = lambda s: s.startswith('http://www.reddit.com/r/')

        data = {}
        data['object'] = sub
        data['type'] = 'Submission'
        data['title'] = clean(sub.title)
        data['text'] = clean(sub.selftext)
        data['created'] = humanize_timestamp(sub.created_utc)
        data['comments'] = '{} comments'.format(sub.num_comments)
        data['score'] = '{} pts'.format(sub.score)
        data['author'] = (clean(sub.author.name) if getattr(sub, 'author')
                          else '[deleted]')
        data['permalink'] = clean(sub.permalink)
        data['subreddit'] = strip_subreddit_url(sub.permalink)
        data['url'] = ('(selfpost)' if is_selfpost(sub.url) else clean(sub.url))

        return data


class SubmissionContent(BaseContent):
    """
    Grab a submission from PRAW and lazily store comments to an internal
    list for repeat access.
    """

    def __init__(
            self,
            submission,
            loader=default_loader,
            indent_size=2,
            max_indent_level=4):

        self.indent_size = indent_size
        self.max_indent_level = max_indent_level
        self._loader = loader
        self._submission = submission
        self._submission_data = None
        self._comment_data = None
        self.name = None

        self.reset()

    @classmethod
    def from_url(
            cls,
            reddit,
            url,
            loader=default_loader,
            indent_size=2,
            max_indent_level=4):

        try:
            with loader():
                submission = reddit.get_submission(url)

        except praw.errors.APIException:
            raise SubmissionURLError(url)

        return cls(submission, loader, indent_size, max_indent_level)

    def reset(self):

        with self._loader():
            self._submission.refresh()
            self._submission_data = self.strip_praw_submission(self._submission)
            self.name = self._submission_data['permalink']
            comments = self.flatten_comments(self._submission.comments)
            self._comment_data = [self.strip_praw_comment(c) for c in comments]

    def get(self, index, n_cols=70):
        """
        Grab the `i`th submission, with the title field formatted to fit inside
        of a window of width `n`
        """

        if index < -1:
            raise IndexError

        elif index == -1:
            data = self._submission_data
            data['split_title'] = textwrap.wrap(data['title'], width=n_cols-2)
            data['split_text'] = textwrap.wrap(data['text'], width=n_cols-2)
            data['n_rows'] = (len(data['split_title']) + len(data['split_text']) + 5)
            data['offset'] = 0

        else:
            data = self._comment_data[index]
            indent_level = min(data['level'], self.max_indent_level)
            data['offset'] = indent_level * self.indent_size

            if data['type'] == 'Comment':
                data['split_body'] = textwrap.wrap(
                    data['body'], width=n_cols-data['offset'])
                data['n_rows'] = len(data['split_body']) + 1
            else:
                data['n_rows'] = 1

        return data

    def toggle(self, index, n_cols=70):
        """
        Toggle the state of the object at the given index.

        If it is a comment, pack it into a hidden comment.
        If it is a hidden comment, unpack it.
        If it is more comments, load the comments.
        """
        data = self.get(index)

        if data['type'] == 'Submission':
            # Can't hide the submission!
            pass

        elif data['type'] == 'Comment':
            cache = [data]
            count = 1
            for d in self.iterate(index+1, 1, n_cols):
                if d['level'] <= data['level']:
                    break

                count += d.get('count', 1)
                cache.append(d)

            comment = {}
            comment['type'] = 'HiddenComment'
            comment['cache'] = cache
            comment['count'] = count
            comment['level'] = data['level']
            comment['body'] = 'Hidden'.format(count)
            self._comment_data[index:index+len(cache)] = [comment]

        elif data['type'] == 'HiddenComment':
            self._comment_data[index:index+1] = data['cache']

        elif data['type'] == 'MoreComments':
            with self._loader():
                comments = data['object'].comments()
                comments = self.flatten_comments(comments, root_level=data['level'])
                comment_data = [self.strip_praw_comment(c) for c in comments]
                self._comment_data[index:index+1] = comment_data

        else:
            raise ValueError('% type not recognized' % data['type'])


class SubredditContent(BaseContent):
    """
    Grabs a subreddit from PRAW and lazily stores submissions to an internal
    list for repeat access.
    """

    def __init__(self, name, submissions, loader=default_loader):

        self.name = name
        self._loader = loader
        self._submissions = submissions
        self._submission_data = []

    @classmethod
    def from_name(cls, reddit, name, loader=default_loader):
        
        display_type = 'normal'

        if name == 'front':
            return cls('Front Page', reddit.get_front_page(limit=None), loader)

        if name == 'all':
            sub = reddit.get_subreddit(name)
        
        else:
            
            if '/' in name:
                name, display_type = name.split('/')

            try:
                with loader():
                    sub = reddit.get_subreddit(name, fetch=True)
            except praw.errors.ClientException:
                raise SubredditNameError(name)
        
        if display_type == 'top':
            return cls('/r/'+sub.display_name+'/top', sub.get_top_from_all(limit=None), loader)

        elif display_type == 'new':
            return cls('/r/'+sub.display_name+'/new', sub.get_new(limit=None), loader)

        else:
            return cls('/r/'+sub.display_name, sub.get_hot(limit=None), loader)

    def get(self, index, n_cols=70):
        """
        Grab the `i`th submission, with the title field formatted to fit inside
        of a window of width `n`
        """

        if index < 0:
            raise IndexError

        while index >= len(self._submission_data):

            try:
                with self._loader():
                    submission = next(self._submissions)
            except StopIteration:
                raise IndexError
            else:
                data = self.strip_praw_submission(submission)
                self._submission_data.append(data)

        # Modifies the original dict, faster than copying
        data = self._submission_data[index]
        data['split_title'] = textwrap.wrap(data['title'], width=n_cols)
        data['n_rows'] = len(data['split_title']) + 3
        data['offset'] = 0

        return data



import argparse
import praw
from requests.exceptions import ConnectionError, HTTPError

from rtv.errors import SubmissionURLError, SubredditNameError
from rtv.utils import curses_session
from rtv.subreddit import SubredditPage
from rtv.submission import SubmissionPage

description = """
Reddit Terminal Viewer is a lightweight browser for www.reddit.com built into a
terminal window.
"""

# TODO: Figure out a way to sync this with the README
epilog = """
Usage
-----
RTV currently supports browsing both subreddits and individual submissions.
In each mode the controls are slightly different. In subreddit mode you can 
browse through the top submissions on either the front page or a specific 
subreddit. In submission mode you can view the self text for a submission and 
browse comments.

Global Commands
  Arrow Keys or `hjkl`: RTV supports both the arrow keys and vim bindings for
                        navigation. Move up and down to scroll through items
                        on the page.
  `r` or `F5`         : Refresh the current page.
  `q`                 : Quit the program.

Subreddit Mode
  Right or `Enter`    : Open the currently selected submission in a new page.
  `/`                 : Open a prompt to switch to a different subreddit. For
                        example, pressing `/` and typing "python" will open
                        "/r/python". You can return to Reddit's front page
                        by using the alias "/r/front".

Submission Mode
  Right or `Enter`    : Toggle the currently selected comment between hidden
                        and visible. Alternatively, load addition comments
                        identified by "[+] more comments".
  Left                : Exit the submission page and return to the subreddit.
"""

def main():

    parser = argparse.ArgumentParser(
        prog='rtv', description=description, epilog=epilog,
        formatter_class=argparse.RawDescriptionHelpFormatter)
    parser.add_argument('-s', dest='subreddit', default='front', help='subreddit name')
    parser.add_argument('-l', dest='link', help='full link to a submission')

    group = parser.add_argument_group(
        'authentication (optional)', 
        'Authenticating allows you to view your customized front page. '
        'If only the username is given, the password will be prompted '
        'securely.')
    group.add_argument('-u', dest='username', help='reddit username')
    group.add_argument('-p', dest='password', help='reddit password')

    args = parser.parse_args()

    try:
        reddit = praw.Reddit(user_agent='reddit terminal viewer v0.0')
        reddit.config.decode_html_entities = True

        if args.username and args.password:
            reddit.login(args.username, args.password)

        with curses_session() as stdscr:

                if args.link:
                    page = SubmissionPage(stdscr, reddit, url=args.link)
                    page.loop()

                page = SubredditPage(stdscr, reddit, args.subreddit)
                page.loop()

    except KeyboardInterrupt:
        return

    except ConnectionError:
        print('Timeout: Could not connect to website')

    except HTTPError:
        print('HTTP Error: 404 Not Found')

    except SubmissionURLError as e:
        print('Could not reach submission URL: {}'.format(e.url))

    except SubredditNameError as e:
        print('Could not reach subreddit: {}'.format(e.name))


if __name__ == '__main__':
    main()



import os
import curses
import time
import threading
from curses import textpad
from contextlib import contextmanager

from .errors import EscapePressed

ESCAPE = 27

class Color(object):

    COLORS = {
        'RED': (curses.COLOR_RED, -1),
        'GREEN': (curses.COLOR_GREEN, -1),
        'YELLOW': (curses.COLOR_YELLOW, -1),
        'BLUE': (curses.COLOR_BLUE, -1),
        'MAGENTA': (curses.COLOR_MAGENTA, -1),
        'CYAN': (curses.COLOR_CYAN, -1),
        'WHITE': (curses.COLOR_WHITE, -1),
        }

    @classmethod
    def get_level(cls, level):

        levels = [cls.MAGENTA, cls.CYAN, cls.GREEN, cls.YELLOW]
        return levels[level % len(levels)]

    @classmethod
    def init(cls):
        """
        Initialize color pairs inside of curses using the default background.

        This should be called once during the curses initial setup. Afterwards,
        curses color pairs can be accessed directly through class attributes.
        """

        # Assign the terminal's default (background) color to code -1
        curses.use_default_colors()

        for index, (attr, code) in enumerate(cls.COLORS.items(), start=1):
            curses.init_pair(index, code[0], code[1])
            setattr(cls, attr, curses.color_pair(index))


def text_input(window):
    """
    Transform a window into a text box that will accept user input and loop
    until an escape sequence is entered.

    If enter is pressed, return the input text as a string.
    If escape is pressed, return None.
    """

    window.clear()
    curses.curs_set(2)
    textbox = textpad.Textbox(window, insert_mode=True)

    def validate(ch):
        "Filters characters for special key sequences"
        if ch == ESCAPE:
            raise EscapePressed
        return ch

    # Wrapping in an exception block so that we can distinguish when the user
    # hits the return character from when the user tries to back out of the
    # input.
    try:
        out = textbox.edit(validate=validate)
        out = out.strip()
    except EscapePressed:
        out = None

    curses.curs_set(0)
    return out


def display_message(stdscr, message):
    "Display a message box at the center of the screen and wait for a keypress"

    message_len = len(message)
    n_rows, n_cols = stdscr.getmaxyx()
    s_row = (n_rows - 2) // 2
    s_col = (n_cols - message_len - 1) // 2
    window = stdscr.derwin(3, message_len+2, s_row, s_col)

    window.erase()
    window.border()
    window.addstr(1, 1, message)
    window.refresh()

    stdscr.getch()

    window.clear()
    window = None
    stdscr.refresh()


class LoadScreen(object):

    def __init__(self, stdscr):

        self._stdscr = stdscr

        self._args = None
        self._animator = None
        self._is_running = None

    def __call__(
            self,
            delay=0.5,
            interval=0.4,
            message='Downloading',
            trail='...'):

        self._args = (delay, interval, message, trail)
        return self

    def __enter__(self):

        self._animator = threading.Thread(target=self.animate, args=self._args)
        self._animator.daemon = True

        self._is_running = True
        self._animator.start()

    def __exit__(self, exc_type, exc_val, exc_tb):

        self._is_running = False
        self._animator.join()

    def animate(self, delay, interval, message, trail):

        # Delay before starting animation to avoid wasting resources if the
        # wait time is very short
        start = time.time()
        while (time.time() - start) < delay:
            if not self._is_running:
                return

        message_len = len(message) + len(trail)
        n_rows, n_cols = self._stdscr.getmaxyx()
        s_row = (n_rows - 2) // 2
        s_col = (n_cols - message_len - 1) // 2
        window = self._stdscr.derwin(3, message_len+2, s_row, s_col)

        while True:
            for i in range(len(trail)+1):

                if not self._is_running:
                    window.clear()
                    window = None
                    self._stdscr.refresh()
                    return

                window.erase()
                window.border()
                window.addstr(1, 1, message + trail[:i])
                window.refresh()
                time.sleep(interval)


@contextmanager
def curses_session():

    try:
        # Curses must wait for some time after the Escape key is pressed to see
        # check if it is the beginning of an escape sequence indicating a
        # special key. The default wait time is 1 second, which means that
        # getch() will not return the escape key (27), until a full second
        # after it has been pressed. Turn this down to 25 ms, which is close to
        # what VIM uses.
        # http://stackoverflow.com/questions/27372068
        os.environ['ESCDELAY'] = '25'

        # Initialize curses
        stdscr = curses.initscr()

        # Turn off echoing of keys, and enter cbreak mode,
        # where no buffering is performed on keyboard input
        curses.noecho()
        curses.cbreak()

        # In keypad mode, escape sequences for special keys
        # (like the cursor keys) will be interpreted and
        # a special value like curses.KEY_LEFT will be returned
        stdscr.keypad(1)

        # Start color, too.  Harmless if the terminal doesn't have
        # color; user can test with has_color() later on.  The try/catch
        # works around a minor bit of over-conscientiousness in the curses
        # module -- the error return from C start_color() is ignorable.
        try:
            curses.start_color()
        except:
            pass
        else:
            Color.init()

        # Hide blinking cursor
        curses.curs_set(0)

        yield stdscr

    finally:

        if stdscr is not None:
            stdscr.keypad(0)
            curses.echo()
            curses.nocbreak()
            curses.endwin()


from setuptools import setup
#python setup.py develop --user
#python setup.py develop --user --uninstall

setup(
    name='rtv',
    version='1.0a3',
    description='A simple terminal viewer for Reddit (Reddit Terminal Viewer)',
    long_description=(
        '**Reddit Terminal Viewer (RTV)** is a lightweight browser '
        'for Reddit (www.reddit.com) built into a terminal window. '
        'RTV is built in Python and utilizes the **curses** library. ' 
        'It is compatible with a large range of terminal emulators on '
        'Linux and OSX systems.'),
    url='https://github.com/michael-lazar/rtv',
    author='Michael Lazar',
    author_email='lazar.michael22@gmail.com',
    license='MIT',
    keywords='reddit terminal praw curses',
    packages=['rtv'],
    install_requires=['praw', 'six'],
    entry_points={'console_scripts': ['rtv=rtv.main:main']},
    classifiers=[
        'Intended Audience :: End Users/Desktop',
        'Environment :: Console :: Curses',
        'Operating System :: MacOS :: MacOS X',
        'Operating System :: POSIX',
        'Natural Language :: English',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.0',
        'Programming Language :: Python :: 3.1',
        'Programming Language :: Python :: 3.2',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Topic :: Terminals',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content :: Message Boards',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content :: News/Diary',
        ],
)



class EscapePressed(Exception):
    pass

class SubmissionURLError(Exception):

    def __init__(self, url):
        self.url = url

class SubredditNameError(Exception):

    def __init__(self, name):
        self.name = name


import curses
import sys
from requests.exceptions import HTTPError

from .errors import SubredditNameError
from .page import BasePage
from .submission import SubmissionPage
from .content import SubredditContent
from .utils import LoadScreen, text_input, display_message, Color, ESCAPE

class SubredditPage(BasePage):

    def __init__(self, stdscr, reddit, name):

        self.reddit = reddit
        self.name = name
        self.loader = LoadScreen(stdscr)

        content = SubredditContent.from_name(reddit, name, self.loader)
        super(SubredditPage, self).__init__(stdscr, content)

    def loop(self):

        self.draw()
        while True:
            cmd = self.stdscr.getch()

            if cmd in (curses.KEY_UP, ord('k')):
                self.move_cursor_up()
                self.clear_input_queue()

            elif cmd in (curses.KEY_DOWN, ord('j')):
                self.move_cursor_down()
                self.clear_input_queue()

            # View submission
            elif cmd in (curses.KEY_RIGHT, curses.KEY_ENTER, ord('l')):
                self.open_submission()
                self.draw()

            # Enter edit mode to change subreddit
            elif cmd == ord('/'):
                self.prompt_subreddit()
                self.draw()

            # Refresh page
            elif cmd in (curses.KEY_F5, ord('r')):
                self.refresh_content()
                self.draw()

            elif cmd == curses.KEY_RESIZE:
                self.draw()

            # Quit
            elif cmd == ord('q'):
                sys.exit()

            else:
                curses.beep()

    def refresh_content(self, name=None):

        name = name or self.name

        try:
            self.content = SubredditContent.from_name(
                self.reddit, name, self.loader)

        except (SubredditNameError, HTTPError):
            display_message(self.stdscr, 'Invalid Subreddit')

        else:
            self.nav.page_index, self.nav.cursor_index = 0, 0
            self.nav.inverted = False
            self.name = name

    def prompt_subreddit(self):

        attr = curses.A_BOLD | Color.MAGENTA
        prompt = 'Enter Subreddit: /r/'
        n_rows, n_cols = self.stdscr.getmaxyx()
        self.stdscr.addstr(n_rows-1, 0, prompt, attr)
        self.stdscr.refresh()
        window = self.stdscr.derwin(n_rows-1, len(prompt))
        window.attrset(attr)

        out = text_input(window)
        if out is not None:
            self.refresh_content(name=out)

    def open_submission(self):
        "Select the current submission to view posts"

        submission = self.content.get(self.nav.absolute_index)['object']
        page = SubmissionPage(self.stdscr, self.reddit, submission=submission)
        page.loop()

    @staticmethod
    def draw_item(win, data, inverted=False):

        n_rows, n_cols = win.getmaxyx()
        n_cols -= 1  # Leave space for the cursor in the first column

        # Handle the case where the window is not large enough to fit the data.
        valid_rows = range(0, n_rows)
        offset = 0 if not inverted else -(data['n_rows'] - n_rows)

        n_title = len(data['split_title'])
        for row, text in enumerate(data['split_title'], start=offset):
            if row in valid_rows:
                attr = curses.A_BOLD
                win.addstr(row, 1, text, attr)

        row = n_title + offset
        if row in valid_rows:
            attr = curses.A_UNDERLINE | Color.BLUE
            text = '{url}'.format(**data)
            win.addnstr(row, 1, text, n_cols-1, attr)

        row = n_title + offset + 1
        if row in valid_rows:
            text = '{created} {comments} {score}'.format(**data)
            win.addnstr(row, 1, text, n_cols-1)

        row = n_title + offset + 2
        if row in valid_rows:
            text = '{author}'.format(**data)
            win.addnstr(row, 1, text, n_cols-1, curses.A_BOLD)
            text = ' {subreddit}'.format(**data)
            win.addnstr(text, n_cols - win.getyx()[1], Color.YELLOW)



import curses
import sys

import six

from .content import SubmissionContent
from .page import BasePage
from .utils import LoadScreen, Color, ESCAPE

class SubmissionPage(BasePage):

    def __init__(self, stdscr, reddit, url=None, submission=None):

        self.loader = LoadScreen(stdscr)

        if url is not None:
            content = SubmissionContent.from_url(reddit, url, self.loader)
        elif submission is not None:
            content = SubmissionContent(submission, self.loader)
        else:
            raise ValueError('Must specify url or submission')

        super(SubmissionPage, self).__init__(stdscr, content, page_index=-1)

    def loop(self):

        self.draw()
        while True:
            cmd = self.stdscr.getch()

            if cmd in (curses.KEY_UP, ord('k')):
                self.move_cursor_up()
                self.clear_input_queue()

            elif cmd in (curses.KEY_DOWN, ord('j')):
                self.move_cursor_down()
                self.clear_input_queue()

            # Refresh page
            elif cmd in (curses.KEY_F5, ord('r')):
                self.refresh_content()
                self.draw()

            # Show / hide a comment tree
            elif cmd in (curses.KEY_RIGHT, curses.KEY_ENTER, ord('l')):
                self.toggle_comment()
                self.draw()

            elif cmd == curses.KEY_RESIZE:
                self.draw()

            # Go back
            elif cmd in (ESCAPE, curses.KEY_LEFT, ord('h')):
                break

            # Quit
            elif cmd == ord('q'):
                sys.exit()

            else:
                curses.beep()

    def toggle_comment(self):

        self.content.toggle(self.nav.absolute_index)

    def refresh_content(self):

        self.content.reset()
        self.nav.page_index, self.nav.cursor_index = -1, 0
        self.nav.inverted = False

    def draw_item(self, win, data, inverted=False):

        if data['type'] == 'MoreComments':
            return self.draw_more_comments(win, data)

        elif data['type'] == 'HiddenComment':
            return self.draw_more_comments(win, data)

        elif data['type'] == 'Comment':
            return self.draw_comment(win, data, inverted=inverted)

        else:
            return self.draw_submission(win, data)

    @staticmethod
    def draw_comment(win, data, inverted=False):

        n_rows, n_cols = win.getmaxyx()
        n_cols -= 1

        # Handle the case where the window is not large enough to fit the data.
        valid_rows = range(0, n_rows)
        offset = 0 if not inverted else -(data['n_rows'] - n_rows)

        row = offset
        if row in valid_rows:
            text = '{author}'.format(**data)
            attr = curses.A_BOLD
            attr |= (Color.BLUE if not data['is_author'] else Color.GREEN)
            win.addnstr(row, 1, text, n_cols-1, attr)
            text = ' {score} {created}'.format(**data)
            win.addnstr(text, n_cols - win.getyx()[1])

        n_body = len(data['split_body'])
        for row, text in enumerate(data['split_body'], start=offset+1):
            if row in valid_rows:
                win.addnstr(row, 1, text, n_cols-1)

        # Vertical line, unfortunately vline() doesn't support custom color so
        # we have to build it one chr at a time.
        attr = Color.get_level(data['level'])
        for y in range(n_rows):

            x = 0

            # Nobody pays attention to curses ;(
            # http://bugs.python.org/issue21088
            if (sys.version_info.major,
                sys.version_info.minor, 
                sys.version_info.micro) == (3, 4, 0):
                x, y = y, x
            
            win.addch(y, x, curses.ACS_VLINE, attr)

        return attr | curses.ACS_VLINE

    @staticmethod
    def draw_more_comments(win, data):

        n_rows, n_cols = win.getmaxyx()
        n_cols -= 1
        text = '{body}'.format(**data)
        win.addnstr(0, 1, text, n_cols-1)
        text = ' [{count}]'.format(**data)
        win.addnstr(text, n_cols - win.getyx()[1], curses.A_BOLD)

        attr = Color.get_level(data['level'])
        for y in range(n_rows):
            win.addch(y, 0, curses.ACS_VLINE, attr)

        return attr | curses.ACS_VLINE

    @staticmethod
    def draw_submission(win, data):

        n_rows, n_cols = win.getmaxyx()
        n_cols -= 3 # one for each side of the border + one for offset

        # Don't print at all if there is not enough room to fit the whole sub
        if data['n_rows'] > n_rows:
            win.addnstr(0, 0, '(Not enough space to display)', n_cols)
            return

        for row, text in enumerate(data['split_title'], start=1):
            win.addnstr(row, 1, text, n_cols, curses.A_BOLD)

        row = len(data['split_title']) + 1
        attr = curses.A_BOLD | Color.GREEN
        text = '{author}'.format(**data)
        win.addnstr(row, 1, text, n_cols, attr)
        text = ' {created} {subreddit}'.format(**data)
        win.addnstr(text, n_cols - win.getyx()[1])

        row = len(data['split_title']) + 2
        attr = curses.A_UNDERLINE | Color.BLUE
        text = '{url}'.format(**data)
        win.addnstr(row, 1, text, n_cols, attr)

        offset = len(data['split_title']) + 3
        for row, text in enumerate(data['split_text'], start=offset):
            win.addnstr(row, 1, text, n_cols)

        row = len(data['split_title']) + len(data['split_text']) + 3
        text = '{score} {comments}'.format(**data)
        win.addnstr(row, 1, text, n_cols, curses.A_BOLD)

        win.border()
